# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: filesystem/filesystem.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x66ilesystem/filesystem.proto\x12\nfilesystem\x1a\x1fgoogle/protobuf/timestamp.proto\"G\n\x0bMoveRequest\x12\x16\n\x06source\x18\x01 \x01(\tR\x06source\x12 \n\x0b\x64\x65stination\x18\x02 \x01(\tR\x0b\x64\x65stination\";\n\x0cMoveResponse\x12+\n\x05\x65ntry\x18\x01 \x01(\x0b\x32\x15.filesystem.EntryInfoR\x05\x65ntry\"$\n\x0eMakeDirRequest\x12\x12\n\x04path\x18\x01 \x01(\tR\x04path\">\n\x0fMakeDirResponse\x12+\n\x05\x65ntry\x18\x01 \x01(\x0b\x32\x15.filesystem.EntryInfoR\x05\x65ntry\"#\n\rRemoveRequest\x12\x12\n\x04path\x18\x01 \x01(\tR\x04path\"\x10\n\x0eRemoveResponse\"!\n\x0bStatRequest\x12\x12\n\x04path\x18\x01 \x01(\tR\x04path\";\n\x0cStatResponse\x12+\n\x05\x65ntry\x18\x01 \x01(\x0b\x32\x15.filesystem.EntryInfoR\x05\x65ntry\"\xd3\x02\n\tEntryInfo\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12(\n\x04type\x18\x02 \x01(\x0e\x32\x14.filesystem.FileTypeR\x04type\x12\x12\n\x04path\x18\x03 \x01(\tR\x04path\x12\x12\n\x04size\x18\x04 \x01(\x03R\x04size\x12\x12\n\x04mode\x18\x05 \x01(\rR\x04mode\x12 \n\x0bpermissions\x18\x06 \x01(\tR\x0bpermissions\x12\x14\n\x05owner\x18\x07 \x01(\tR\x05owner\x12\x14\n\x05group\x18\x08 \x01(\tR\x05group\x12?\n\rmodified_time\x18\t \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0cmodifiedTime\x12*\n\x0esymlink_target\x18\n \x01(\tH\x00R\rsymlinkTarget\x88\x01\x01\x42\x11\n\x0f_symlink_target\":\n\x0eListDirRequest\x12\x12\n\x04path\x18\x01 \x01(\tR\x04path\x12\x14\n\x05\x64\x65pth\x18\x02 \x01(\rR\x05\x64\x65pth\"B\n\x0fListDirResponse\x12/\n\x07\x65ntries\x18\x01 \x03(\x0b\x32\x15.filesystem.EntryInfoR\x07\x65ntries\"C\n\x0fWatchDirRequest\x12\x12\n\x04path\x18\x01 \x01(\tR\x04path\x12\x1c\n\trecursive\x18\x02 \x01(\x08R\trecursive\"P\n\x0f\x46ilesystemEvent\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12)\n\x04type\x18\x02 \x01(\x0e\x32\x15.filesystem.EventTypeR\x04type\"\xfe\x01\n\x10WatchDirResponse\x12?\n\x05start\x18\x01 \x01(\x0b\x32\'.filesystem.WatchDirResponse.StartEventH\x00R\x05start\x12=\n\nfilesystem\x18\x02 \x01(\x0b\x32\x1b.filesystem.FilesystemEventH\x00R\nfilesystem\x12\x46\n\tkeepalive\x18\x03 \x01(\x0b\x32&.filesystem.WatchDirResponse.KeepAliveH\x00R\tkeepalive\x1a\x0c\n\nStartEvent\x1a\x0b\n\tKeepAliveB\x07\n\x05\x65vent\"H\n\x14\x43reateWatcherRequest\x12\x12\n\x04path\x18\x01 \x01(\tR\x04path\x12\x1c\n\trecursive\x18\x02 \x01(\x08R\trecursive\"6\n\x15\x43reateWatcherResponse\x12\x1d\n\nwatcher_id\x18\x01 \x01(\tR\twatcherId\"8\n\x17GetWatcherEventsRequest\x12\x1d\n\nwatcher_id\x18\x01 \x01(\tR\twatcherId\"O\n\x18GetWatcherEventsResponse\x12\x33\n\x06\x65vents\x18\x01 \x03(\x0b\x32\x1b.filesystem.FilesystemEventR\x06\x65vents\"5\n\x14RemoveWatcherRequest\x12\x1d\n\nwatcher_id\x18\x01 \x01(\tR\twatcherId\"\x17\n\x15RemoveWatcherResponse*R\n\x08\x46ileType\x12\x19\n\x15\x46ILE_TYPE_UNSPECIFIED\x10\x00\x12\x12\n\x0e\x46ILE_TYPE_FILE\x10\x01\x12\x17\n\x13\x46ILE_TYPE_DIRECTORY\x10\x02*\x98\x01\n\tEventType\x12\x1a\n\x16\x45VENT_TYPE_UNSPECIFIED\x10\x00\x12\x15\n\x11\x45VENT_TYPE_CREATE\x10\x01\x12\x14\n\x10\x45VENT_TYPE_WRITE\x10\x02\x12\x15\n\x11\x45VENT_TYPE_REMOVE\x10\x03\x12\x15\n\x11\x45VENT_TYPE_RENAME\x10\x04\x12\x14\n\x10\x45VENT_TYPE_CHMOD\x10\x05\x32\x9f\x05\n\nFilesystem\x12\x39\n\x04Stat\x12\x17.filesystem.StatRequest\x1a\x18.filesystem.StatResponse\x12\x42\n\x07MakeDir\x12\x1a.filesystem.MakeDirRequest\x1a\x1b.filesystem.MakeDirResponse\x12\x39\n\x04Move\x12\x17.filesystem.MoveRequest\x1a\x18.filesystem.MoveResponse\x12\x42\n\x07ListDir\x12\x1a.filesystem.ListDirRequest\x1a\x1b.filesystem.ListDirResponse\x12?\n\x06Remove\x12\x19.filesystem.RemoveRequest\x1a\x1a.filesystem.RemoveResponse\x12G\n\x08WatchDir\x12\x1b.filesystem.WatchDirRequest\x1a\x1c.filesystem.WatchDirResponse0\x01\x12T\n\rCreateWatcher\x12 .filesystem.CreateWatcherRequest\x1a!.filesystem.CreateWatcherResponse\x12]\n\x10GetWatcherEvents\x12#.filesystem.GetWatcherEventsRequest\x1a$.filesystem.GetWatcherEventsResponse\x12T\n\rRemoveWatcher\x12 .filesystem.RemoveWatcherRequest\x1a!.filesystem.RemoveWatcherResponseBi\n\x0e\x63om.filesystemB\x0f\x46ilesystemProtoP\x01\xa2\x02\x03\x46XX\xaa\x02\nFilesystem\xca\x02\nFilesystem\xe2\x02\x16\x46ilesystem\\GPBMetadata\xea\x02\nFilesystemb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'filesystem.filesystem_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\016com.filesystemB\017FilesystemProtoP\001\242\002\003FXX\252\002\nFilesystem\312\002\nFilesystem\342\002\026Filesystem\\GPBMetadata\352\002\nFilesystem'
  _globals['_FILETYPE']._serialized_start=1690
  _globals['_FILETYPE']._serialized_end=1772
  _globals['_EVENTTYPE']._serialized_start=1775
  _globals['_EVENTTYPE']._serialized_end=1927
  _globals['_MOVEREQUEST']._serialized_start=76
  _globals['_MOVEREQUEST']._serialized_end=147
  _globals['_MOVERESPONSE']._serialized_start=149
  _globals['_MOVERESPONSE']._serialized_end=208
  _globals['_MAKEDIRREQUEST']._serialized_start=210
  _globals['_MAKEDIRREQUEST']._serialized_end=246
  _globals['_MAKEDIRRESPONSE']._serialized_start=248
  _globals['_MAKEDIRRESPONSE']._serialized_end=310
  _globals['_REMOVEREQUEST']._serialized_start=312
  _globals['_REMOVEREQUEST']._serialized_end=347
  _globals['_REMOVERESPONSE']._serialized_start=349
  _globals['_REMOVERESPONSE']._serialized_end=365
  _globals['_STATREQUEST']._serialized_start=367
  _globals['_STATREQUEST']._serialized_end=400
  _globals['_STATRESPONSE']._serialized_start=402
  _globals['_STATRESPONSE']._serialized_end=461
  _globals['_ENTRYINFO']._serialized_start=464
  _globals['_ENTRYINFO']._serialized_end=803
  _globals['_LISTDIRREQUEST']._serialized_start=805
  _globals['_LISTDIRREQUEST']._serialized_end=863
  _globals['_LISTDIRRESPONSE']._serialized_start=865
  _globals['_LISTDIRRESPONSE']._serialized_end=931
  _globals['_WATCHDIRREQUEST']._serialized_start=933
  _globals['_WATCHDIRREQUEST']._serialized_end=1000
  _globals['_FILESYSTEMEVENT']._serialized_start=1002
  _globals['_FILESYSTEMEVENT']._serialized_end=1082
  _globals['_WATCHDIRRESPONSE']._serialized_start=1085
  _globals['_WATCHDIRRESPONSE']._serialized_end=1339
  _globals['_WATCHDIRRESPONSE_STARTEVENT']._serialized_start=1305
  _globals['_WATCHDIRRESPONSE_STARTEVENT']._serialized_end=1317
  _globals['_WATCHDIRRESPONSE_KEEPALIVE']._serialized_start=1319
  _globals['_WATCHDIRRESPONSE_KEEPALIVE']._serialized_end=1330
  _globals['_CREATEWATCHERREQUEST']._serialized_start=1341
  _globals['_CREATEWATCHERREQUEST']._serialized_end=1413
  _globals['_CREATEWATCHERRESPONSE']._serialized_start=1415
  _globals['_CREATEWATCHERRESPONSE']._serialized_end=1469
  _globals['_GETWATCHEREVENTSREQUEST']._serialized_start=1471
  _globals['_GETWATCHEREVENTSREQUEST']._serialized_end=1527
  _globals['_GETWATCHEREVENTSRESPONSE']._serialized_start=1529
  _globals['_GETWATCHEREVENTSRESPONSE']._serialized_end=1608
  _globals['_REMOVEWATCHERREQUEST']._serialized_start=1610
  _globals['_REMOVEWATCHERREQUEST']._serialized_end=1663
  _globals['_REMOVEWATCHERRESPONSE']._serialized_start=1665
  _globals['_REMOVEWATCHERRESPONSE']._serialized_end=1688
  _globals['_FILESYSTEM']._serialized_start=1930
  _globals['_FILESYSTEM']._serialized_end=2601
# @@protoc_insertion_point(module_scope)
